@@ -53,9 +53,9 @@ def det_infer_demo():
     from XEdu.hub import Workflow as wf
     img = 'hand1.jpeg' # 指定进行推理的图片路径
 
-    det = wf(task='det_hand')#,checkpoint='rtmdet-acc0de.onnx')# ,checkpoint="rtmpose-m-80e511.onnx") # 实例化mmpose模型
+    det = wf(task='det_hand',checkpoint='det_fire.onnx')
 
-    bboxs,im_ou = det.inference(data=img,img_type='pil',show=True) # 在CPU上进行推理
+    bboxs,im_ou = det.inference(data=img,img_type='pil',show=False) # 在CPU上进行推理
     # print(bboxs)
     # det.save(im_ou,"im_ou_d.jpg")
 
@@ -108,13 +108,13 @@ def face_det_demo():
     
 
 def ocr_demo():
-    img = 'ocr.jpg' # 指定进行推理的图片路径
-    ocr = wf(task='ocr' ,download_path='ocr_file')#,checkpoint="rtmdet-coco.onnx") # 实例化mmpose模型
+    img = 'cat4.jpg' # 指定进行推理的图片路径
+    ocr = wf(task='ocr1' )#,checkpoint="rtmdet-coco.onnx") # 实例化mmpose模型
 
     result,img = ocr.inference(data=img,img_type='pil',show=True) # 在CPU上进行推理
-    # print(result)
-    # ocr.show(img)
-    ocr.save(img,"pimg_ou.png")
+    print(result)
+    ocr.show(img)
+    ocr.save(img,"img_ou.png")
     
     re = ocr.format_output(lang="zh")
 
@@ -125,11 +125,11 @@ def mmedu_demo():
     # print(result)
     re = mm.format_output(lang="zh")
 
-    mm = wf(task='mmedu',checkpoint="convert_model.onnx")
-    result, img = mm.inference(data='fire.jpg',img_type='pil',thr=0.6)
-    # mm.show(img)
-    # print(result)
-    re = mm.format_output(lang="zh")
+    # mm = wf(task='mmedu',checkpoint="convert_model.onnx")
+    # result, img = mm.inference(data='fire.jpg',img_type='pil',thr=0.6)
+    # # mm.show(img)
+    # # print(result)
+    # re = mm.format_output(lang="zh")
 
 def basenn_demo():
     nn = wf(task='basenn',checkpoint="checkpoints/basenn.onnx") # iris act 
@@ -277,39 +277,37 @@ def drive_demo():
 def embedding_demo():
     # 导入依赖库
     from XEdu.hub import Workflow as wf
-    from XEdu.utils import get_similarity
+    from XEdu.utils import get_similarity,visualize_similarity,visualize_probability
     # 实例化图像嵌入模型
     img_emb = wf(task='embedding_image')
     # 实例化文本嵌入模型
     text_emb = wf(task='embedding_text')
-    images = ["cat.jpg"]
+    images = ["cat.png","cat1.jpg","cat2.jpg","cat3.jpg","ele.jpg","dog.jpg"]
     texts_zh = ["猫", "狗"]
-    texts_en = ["cat", "dog"]
+    texts_en = ["cat", "dog","room","elephant"]
 
-    image_embeddings = img_emb.inference(data=images)
-    text_zh_embeddings = text_emb.inference(data=texts_zh)
+    # image_embeddings = img_emb.inference(data=images)
+    # text_zh_embeddings = text_emb.inference(data=texts_zh)
     text_en_embeddings = text_emb.inference(data=texts_en)
 
-    # logits = get_similarity(image_embeddings,text_zh_embeddings,method='cosine',use_softmax=False)
-    logits = get_similarity(text_en_embeddings,text_zh_embeddings) # ,method='cosine',use_softmax=False)
-    print("cosine", logits)
+    # # 图像 - 文本相似度
+    # logits = get_similarity(image_embeddings,text_en_embeddings,use_softmax=False)
+    # visualize_similarity(logits, images, texts_en,figsize=(10,20),)
+    # visualize_probability(logits, images, texts_en,topk=6,figsize=(10,10))  
 
-    # 导入依赖库
-    from XEdu.hub import Workflow as wf
-    from XEdu.utils import get_similarity
-    # 实例化图像嵌入模型
-    img_emb = wf(task='embedding_image')
-    # 实例化文本嵌入模型
-    text_emb = wf(task='embedding_text')
-    # 示例输入
-    images = ["cat.jpg"]
-    texts = ["猫", "狗"]
-    # 对图像进行编码
-    image_embeddings = img_emb.inference(data=images)
-    # 对文本进行编码
-    text_embeddings = text_emb.inference(data=texts)
-    # 计算相似度
-    similarity = get_similarity(image_embeddings,text_embeddings,method='cosine')
+    # 文本 - 文本相似度
+    logits = get_similarity(text_en_embeddings,text_en_embeddings ,method='cosine',use_softmax=False)
+    visualize_similarity(logits, texts_en, texts_en)
+
+    # # 文本 - 图像相似度
+    # logits = get_similarity(text_en_embeddings,image_embeddings,method='cosine',use_softmax=False)
+    # visualize_similarity(logits, texts_en, images)
+
+    # 图像 - 图像相似度
+    # logits = get_similarity(image_embeddings,images_embeddings,method='cosine',use_softmax=False)
+    # visualize_similarity(logits, images, imagess)
+
+    print("cosine", logits)
 
 
 def openvino_demo():
@@ -398,10 +396,204 @@ def color_demo():
     from XEdu.hub import Workflow as wf
     color = wf(task='gen_color',download_path='new_download')# ,checkpoint='/home/user/下载/colorization-master/colorizer_siggraph17.onnx')
     img = cv2.imread('/home/user/下载/colorization-master/imgs/ansel_adams3.jpg')
-    result = color.inference(data='/home/user/下载/colorization-master/imgs/ansel_adams3.jpg',img_type='pil')
+    result = color.inference(data='/home/user/下载/colorization-master/imgs/ansel_adams3.jpg',img_type='pil',show=True)
+    color.show(result)
+    color.format_output(lang="zh")
     color.save(result,"color_ou.jpg")
 
 
+def kimi_stream_demo():
+    import os
+    from openai import OpenAI
+    
+    client = OpenAI(
+        api_key="sk-c1nBfFK5mcwtilUBr6uvAVulwUhIBWNOqrbVZstvmwBXUcI9",
+        base_url="https://api.moonshot.cn/v1",
+    )
+    
+    response = client.chat.completions.create(
+        model="moonshot-v1-8k",
+        messages=[
+            {
+                "role": "system",
+                "content": "你是 Kimi，由 Moonshot AI 提供的人工智能助手，你更擅长中文和英文的对话。你会为用户提供安全，有帮助，准确的回答。同时，你会拒绝一切涉及恐怖主义，种族歧视，黄色暴力等问题的回答。Moonshot AI 为专有名词，不可翻译成其他语言。",
+            },
+            {"role": "user", "content": "你好，我叫李雷，1+1等于多少？"},
+        ],
+        temperature=0.3,
+        stream=True,
+    )
+    
+    collected_messages = []
+    for idx, chunk in enumerate(response):
+        # print("Chunk received, value: ", chunk)
+        chunk_message = chunk.choices[0].delta
+        if not chunk_message.content:
+            continue
+        collected_messages.append(chunk_message)  # save the message
+        print(f"#{idx}: {''.join([m.content for m in collected_messages])}")
+    print(f"Full conversation received: {''.join([m.content for m in collected_messages])}")
+
+def  llm_stream():
+    import json
+    import requests
+
+    url = "https://api.moonshot.cn/v1"  # 替换为目标URL
+    data = {
+        "stream": True,
+        "model": "moonshot-v1-8k",
+        "messages": [
+            {
+                "role": "user",
+                "content": "你是谁"
+            },
+        ]
+    }
+    headers = {
+        "Authorization": "Bearer sk-c1nBfFK5mcwtilUBr6uvAVulwUhIBWNOqrbVZstvmwBXUcI9",
+    }
+
+    # 使用 with 语句确保请求完成后释放资源
+    with requests.post(url, json=data, headers=headers, timeout=60000, stream=True) as response:
+        # print(response.headers)
+        for chunk in response.iter_lines(chunk_size=None):
+            mChunk = chunk.decode('utf-8')
+            if "[DONE]" in mChunk:
+                break
+            print(mChunk)
+            lines = mChunk.splitlines()
+            for line in lines:
+                respStr = line.strip().replace("data: ", "")
+                respContent = ""
+                try:
+                    respJson = json.loads(respStr)
+                    respContent = respJson["choices"][0]["delta"]["content"]
+                except  Exception as e:
+                    respContent = ""
+                print(respContent)
+
+# 存储各种服务的API Key和URL
+class ServiceProvider:
+    BASE_URL = {
+        "openrouter":"https://openrouter.ai/api/v1/chat/completions",
+        "moonshot":"https://api.moonshot.cn/v1/chat/completions",
+        "google": "https://generativelanguage.googleapis.com/v1beta/models/gemini-pro:generateContent?key=AIzaSyBY-2VWbVvWybt8qjnruUY4tip4X_OwCp8"
+
+    }
+    API_KEY = {
+        "openrouter": "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",
+        "moonshot": "sk-c1nBfFK5mcwtilUBr6uvAVulwUhIBWNOqrbVZstvmwBXUcI9",
+        "google": "AIzaSyBY-2VWbVvWybt8qjnruUY4tip4X_OwCp8"
+    }
+    MODEL = {
+        "openrouter": "mistralai/mistral-7b-instruct:free",
+        "moonshot": "moonshot-v1-8k",
+        "google": "gemini-pro"
+    }
+    def __init__(self, service_name):
+        self.api_key = ServiceProvider.API_KEY[service_name]
+        self.url = ServiceProvider.BASE_URL[service_name]
+        self.model = ServiceProvider.MODEL[service_name]
+import requests
+import json
+class Client:
+    """docstring for Client
+    
+    """
+
+    def __init__(self, provider=None,base_url=None,api_key=None,model=None):
+        self.provider = provider
+        self.base_url = base_url
+        self.api_key = api_key
+        self.model = model
+
+        if provider:
+            self.api_key = ServiceProvider.API_KEY[provider]
+            self.url = ServiceProvider.BASE_URL[provider]
+            self.model = ServiceProvider.MODEL[provider]
+    
+    def inference(self, message, stream=False):
+        if isinstance(message,str):
+            messages = [{"role": "user", "content": message}]
+        else:
+            messages = message
+        if stream:
+            data = json.dumps({
+                        "model": self.model, # Optional
+                        "messages": messages,
+                        "stream": stream,
+                    })
+        else:
+            data = json.dumps({
+                        "model": self.model, # Optional
+                        "messages": messages,
+                    })
+        response = requests.post(
+            url=self.url,
+            headers={
+                'Accept': 'text/event-stream',
+                "Authorization": f"Bearer {self.api_key}",
+                "Content-Type": "application/json",
+            },
+            data=data
+            # stream=stream,
+        )
+        if stream:
+            content = ''
+            for output in response.iter_lines(chunk_size=None):
+                # print("+++++++++++",output)
+                if output==b'data: [DONE]':
+                    break
+                # 判断content是否为空
+                elif output[:6] == b'data: ':
+                    # print("------------",output[6:])
+                    data = output[6:]
+                    output = json.loads(data)
+                    content = output["choices"][0]["delta"]["content"]# .encode('latin1').decode('utf-8')
+        else:
+            try:
+                content = response.json()["choices"][0]["message"]
+            except Exception as e:
+                content = response.json()
+
+        print(content)
+        return content
+
+def llm_demo(stream=False):
+    from XEdu.LLM import Client
+    print(Client.support_provider())
+    # client = Client(provider="openrouter",api_key="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")
+    client = Client(base_url="https://openrouter.ai/api/v1",api_key="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",model="mistralai/mistral-7b-instruct:free")
+    # client = Client(provider="moonshot",api_key="sk-c1nBfFK5mcwtilUBr6uvAVulwUhIBWNOqrbVZstvmwBXUcI9")
+    # client = Client(provider="deepseek",api_key="sk-bc3da8219e624f52a80b565a6aa31dcd")
+    # client = Client(provider="glm",api_key="03a6c63bc0d356c5819f20b009b511e9.EMc2bCVDb7XJYuJf")
+    # client = Client(provider="replicate",api_key="r8_HjFF0greljF48VQUFpCOUF021NSciu62QhRZP") # todo
+    # print(client.support_model())
+    # detail = client.model_info("deepseek-chat")
+    # print(detail)
+    res = client.inference("你好,用中文介绍一下你自己",stream=True)
+
+def seg_demo():
+    from XEdu.hub import Workflow as wf
+    seg = wf(task='seg_sam') # iris act 
+    img_p = '/home/user/下载/sam/efficientvit/assets/fig/cat.jpg'
+    # img = 'ele.jpg'
+    # img = 'fire.jpg'
+    result, img = seg.inference(data=img_p,img_type='cv2',show=True,prompt=[[170,70],[380,270]])
+    result, img = seg.inference(data=img_p,img_type='cv2',show=True,prompt=[[380,270]])
+    seg.show(img)
+    import time
+    start = time.time()
+    result,img = seg.inference(data=img_p,mode='box',prompt=[170,70,380,270],img_type='cv2',show=True)
+    result,img = seg.inference(data=img_p,mode='box',prompt=[[170,70,380,270]],img_type='cv2',show=True)
+    result,img = seg.inference(data=img_p,mode='box',prompt=[[170,70,380,270]],img_type='cv2',show=True)
+    print(time.time()-start)
+    # print(result.shape)
+    seg.show(result[0])
+    seg.show(img)
+    seg.format_output(lang="zh")
+    seg.save(img,"seg_ou.png")
+
 if __name__ == "__main__":
     # pose_infer_demo()
     # det_infer_demo()
@@ -420,11 +612,11 @@ if __name__ == "__main__":
     # qa_demo()
     # drive_demo()
     # embedding_demo()
-    color_demo()
-    # from openxlab.model import download
-    # download(model_repo='xedu/hub-model', model_name='gen_color.onnx')
-    # url = "https://download.openxlab.org.cn/repos/file/xedu/hub-model/main?filepath=gen_color.onnx&sign=2838feb49073bb8f756d57a9f78a68ab&nonce=1713425582020"
-    # path = "git-lfs/gen_color.onnx"
-    # downloader = Downloader(url, path)
-    # downloader.start()
+    # color_demo()
+    # llm_demo()
+    # client = Client(provider="openrouter")
+    # client.inference("你好,用中文介绍一下你自己")
+    # llm_stream()
+    seg_demo()
+
 
