@@ -19,7 +19,9 @@ from .tokenizer.qa_squadprocess import *
 from .tokenizer.clip_tokenizer import Preprocessor, Tokenizer
 from .datatset_class import coco_class,imagenet_class
 import copy
-
+import requests
+from tqdm import tqdm
+from .models import efficientVIT_sam as sam 
 from .errorcode import ErrorCodeFactory as ecf
 T = TypeVar("T")
 
@@ -73,14 +75,50 @@ task_dict = {
         "drive_perception":"drive_perception.onnx",
         "embedding_image":"embedding_image.onnx",
         "embedding_text":"embedding_text.onnx",
+        "gen_color":"gen_color.onnx",
+        "seg_sam":['seg_sam_encoder.onnx','seg_sam_decoder.onnx'],
         "det_face":"...",
         "ocr":"...",
         "mmedu":"...",
         "basenn":"...",
         "baseml":"...",
+        "custom":"...",
 }
 style_list = ['mosaic','candy','rain-princess','udnie','pointilism']
 
+class Downloader(object):
+    def __init__(self, url, file_path,model_name=None,output_dir='checkpoints',overwrite=False):
+        self.url = url
+        self.file_path = file_path
+        if model_name is None:
+            self.model_name = os.path.basename(file_path)
+        else:
+            self.model_name = model_name
+        self.output_dir = output_dir
+
+    def start(self):
+        res_length = requests.get(self.url, stream=True)
+        total_size = int(res_length.headers['Content-Length'])
+        if os.path.exists(self.file_path):
+            temp_size = os.path.getsize(self.file_path)
+        else:
+            temp_size = 0
+        print(f"model_repo:xedu/hub-model, model_name:{self.model_name}, output:{self.output_dir}, overwrite:False")
+        headers = {'Range': 'bytes=%d-' % temp_size,
+                   "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:81.0) Gecko/20100101 Firefox/81.0"}
+        res_left = requests.get(self.url, stream=True, headers=headers)
+        total_size_in_bytes = int(res_left.headers['Content-Length'])
+        progress_bar = tqdm(total=total_size_in_bytes, unit='iB', unit_scale=True)
+
+        with open(self.file_path, "ab") as f:
+            for chunk in res_left.iter_content(chunk_size=1024):
+                temp_size += len(chunk)
+                progress_bar.update(len(chunk))
+                f.write(chunk)
+                
+        progress_bar.close()
+        print(f"download model repo:xedu/hub-model, file_name:{self.model_name} ")
+
 class Workflow:
     """
         Workflow类用于加载预训练模型以解决各类任务。
@@ -156,6 +194,8 @@ class Workflow:
             "bodydetect":"det_body",
         }
         path = download_path if download_path is not None else "checkpoints"
+        if task not in self.task_dict.keys() and task not in self.task_nick_name.keys():
+            raise ValueError(f"Error Code: -310. No such task: '{task}'. Please refer to 'support_task()' method for a list of supported tasks. ")
         if task in self.task_nick_name.keys():
             self.task = self.task_nick_name[task]
         else:
@@ -179,7 +219,8 @@ class Workflow:
             self.questions = []
             self.contexts = []
             self.doc = None
-        default_task = ['det_body','det_body_l','det_coco','det_coco_l','pose_body17','pose_body17_l','pose_body26','pose_wholebody133','pose_face106','pose_hand21','det_hand','cls_imagenet','gen_style','nlp_qa','drive_perception','embedding_image','embedding_text']
+        default_task = ['det_body','det_body_l','det_coco','det_coco_l','pose_body17','pose_body17_l','pose_body26','pose_wholebody133','pose_face106','pose_hand21','det_hand','cls_imagenet','gen_style','nlp_qa','drive_perception','embedding_image','embedding_text','gen_color']
+        
         if checkpoint is None and self.task in default_task: # 若不指定权重文件，则使用对应任务的默认模型
             if self.task == 'gen_style':
                 self.style = kwargs.get('style',None)
@@ -196,7 +237,7 @@ class Workflow:
             else:
                 checkpoint = self.task_dict[self.task]
             checkpoint = os.path.join(path, checkpoint)
-            if not os.path.exists(checkpoint): # 下载默认模型
+            if not os.path.exists(checkpoint): # 本地未检测到模型，云端下载默认模型
                 print("本地未检测到{}任务对应模型，云端下载中...".format(self.task))
                 # path = "checkpoints"
                 if not os.path.exists(path):
@@ -226,12 +267,16 @@ class Workflow:
                     "embedding_image":"clip_image",
                     "embedding_text":"clip_text",
                 }
-                # download('test12318/bert-english',model_name=model_name,output=path)
-                download(model_repo='xedu/hub-model', model_name=model_name_map[self.task],output=path)
+                model_name_map_download = {
+                    "gen_color":"https://download.openxlab.org.cn/repos/file/xedu/hub-model/main?filepath=gen_color.onnx&sign=2838feb49073bb8f756d57a9f78a68ab&nonce=1713425582020",
+                }
+                if self.task in model_name_map_download.keys():
+                    downloader = Downloader(model_name_map_download[self.task], os.path.join(path, self.task_dict[self.task]),output_dir=path)
+                    downloader.start()
+                else:
+                    download(model_repo='xedu/hub-model', model_name=model_name_map[self.task],output=path)
             # download()
             self.model = ort.InferenceSession(checkpoint, None)
-
-        # model_path='rtmpose-ort/rtmpose-s-0b29a8.onnx'
         elif self.task =="det_face":
                 self.model = cv2.CascadeClassifier(cv2.data.haarcascades+'haarcascade_frontalface_default.xml')
         elif self.task == "ocr":
@@ -247,7 +292,6 @@ class Workflow:
                 return
             assert os.path.exists(checkpoint),ecf.NO_SUCH_CHECKPOINT(checkpoint)
             assert os.path.splitext(checkpoint)[-1]==".onnx",ecf.CHECKPOINT_TYPE(os.path.splitext(checkpoint)[-1].strip("."))
-
             from .BaseDeploy import _BaseDeploy as bd
             self.model = bd(checkpoint)
         elif self.task == 'baseml':
@@ -273,12 +317,37 @@ class Workflow:
                     pass
             else:
                 self.model = model
+        elif  self.task == 'seg_sam':
+            checkpoint = [os.path.join(path, task_dict[self.task][0]),os.path.join(path, task_dict[self.task][1])]
+            if not os.path.exists(checkpoint[0]):
+                print("本地未检测到{}任务对应encoder模型，云端下载中...".format(self.task))
+                if not os.path.exists(path):
+                    os.mkdir(path)
+                downloader = Downloader("https://download.openxlab.org.cn/repos/file/xedu/hub-model/main?filepath=seg_sam_encoder.onnx&sign=e56fc1c91d73d75d42c32b95a34ac42b&nonce=1717570798621", checkpoint[0],output_dir=path)
+                downloader.start()
+            if not os.path.exists(checkpoint[1]):
+                print("本地未检测到{}任务对应decoder模型，云端下载中...".format(self.task))
+                downloader = Downloader("https://download.openxlab.org.cn/repos/file/xedu/hub-model/main?filepath=seg_sam_decoder.onnx&sign=2c8881dc025032d523322e9ee5289a63&nonce=1717570769188", checkpoint[1],output_dir=path)
+                downloader.start()
+            self.model = [sam.SamEncoder(model_path=checkpoint[0]),sam.SamDecoder(model_path=checkpoint[1])]
         else:
             assert os.path.exists(checkpoint),ecf.NO_SUCH_CHECKPOINT(checkpoint)
             assert os.path.splitext(checkpoint)[-1]==".onnx",ecf.CHECKPOINT_TYPE(os.path.splitext(checkpoint)[-1].strip("."))
             self.model = ort.InferenceSession(checkpoint, None)
+        if self.task[:3] in ['det','cls']:
+            self._check_mmedu(self.model)
         print(f"{self.task}任务模型加载成功！")
     
+    def _check_mmedu(self,model):
+        model_meta = model.get_modelmeta()
+        key='MODEL_INFO'
+        if key in model_meta.custom_metadata_map:
+            unicode_string = model_meta.custom_metadata_map[key]
+            # 不匹配的任务名
+            info = f'Error code: -311. Input task type "{self.task}" does not match the model which is generated by MMEdu. Please set task="mmedu" instead of "{self.task}".'
+            raise ValueError(info)
+            # print(f'Model info: {unicode_string[:100] + "..." if len(unicode_string) > 100 else unicode_string}')
+            
 
     def inference(self,data=None,show=False,img_type=None,thr=0.3,bbox=None,target_class=None,erase=False,preprocess=None, postprocess=None,**kwargs):
         self._get_img = img_type
@@ -338,7 +407,161 @@ class Workflow:
             self._tokenizer = Tokenizer()
             self._batch_size = 16
             return self._embedding_text_infer(data)
-    
+        elif self.task in ['gen_color']:
+            return self._gen_color_infer(data,show,img_type)
+        elif self.task in ['seg_sam']:
+            return self._seg_sam_infer(data,show,img_type,**kwargs)
+
+    def _seg_sam_infer(self,data,show,img_type,mode='point',prompt=None):
+        if isinstance(data,str):
+            assert os.path.exists(data),ecf.NO_SUCH_FILE(data)
+            np_data = cv2.imread(data)
+        else:
+            np_data = data
+
+        raw_img = cv2.cvtColor(np_data, cv2.COLOR_BGR2RGB)
+        origin_image_size = raw_img.shape[:2]
+        img = sam.preprocess(raw_img, img_size=512)
+
+        img_embeddings = self.model[0](img) # encoder
+        if mode == "point":
+            if prompt is None:
+                prompt = np.array([origin_image_size[1] /2,origin_image_size[0]/2,1],dtype='float32')
+            # point = np.concatenate([prompt, [1]],dtype='float32')[None,None,:]
+            ax = [[1] for i in range(len(prompt))]
+            point = np.concatenate([prompt, ax],axis=1)
+            point = np.array([point],dtype='float32')
+            point_coords = point[..., :2]
+            point_labels = point[..., 2]
+            masks, self.mask_scores, _ = self.model[1].run(
+                img_embeddings=img_embeddings,
+                origin_image_size=origin_image_size,
+                point_coords=point_coords,
+                point_labels=point_labels,
+            ) # decoder
+            self.masks = masks[0]
+        elif mode == "box":
+            if len(np.array(prompt).shape) == 1:
+                boxes = np.array(prompt)[None,:]
+            else:
+                boxes = np.array(prompt)
+            masks, self.mask_scores, _ = self.model[1].run(
+                img_embeddings=img_embeddings,
+                origin_image_size=origin_image_size,
+                boxes=boxes,
+            ) # decoder
+            self.masks = masks[0]
+        else:
+            raise NotImplementedError
+        if img_type is None:
+            self._get_img = "pil"
+        else:
+            self._get_img = img_type
+            # 增加alpha通道
+            # raw_img = cv2.cvtColor(raw_img, cv2.COLOR_BGR2BGRA)
+            w, h, _ = raw_img.shape
+
+            fig = plt.figure(figsize=(h/100,w/100), dpi=100)
+            plt.imshow(raw_img)
+            if mode == "point":
+                for mask in masks[0]:
+                    sam.show_mask(mask, plt.gca(), random_color=len(masks) > 1)
+                sam.show_points(point_coords, point_labels, plt.gca())
+            elif mode == "box":
+                for mask in masks[0]:
+                    sam.show_mask(mask, plt.gca(), random_color=len(masks) > 1)
+                for box in boxes:
+                    sam.show_box(box, plt.gca())
+
+            plt.axis("off")
+            plt.tight_layout()
+            plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
+            plt.margins(0, 0)
+            # 将画布保存到字节流
+            buf = io.BytesIO()
+            plt.savefig(buf, dpi=fig.dpi, format='png', bbox_inches='tight', pad_inches=0)
+            buf.seek(0)
+
+            # 使用PIL从字节流中加载图像并转换为数组
+            img = Image.open(buf)
+            return_img = np.array(img)
+            if img_type == 'cv2':
+                return_img = cv2.cvtColor(return_img,cv2.COLOR_RGB2BGR)
+            plt.close(fig)
+            if show:
+                # fig = plt.figure(figsize=(10,10))
+                self.show(return_img)
+            # print("data",img.shape)
+            return masks[0], return_img
+        return masks[0]
+        
+    def _gen_color_infer(self,data=None,show=False,img_type=None):
+
+        def load_img(img_path):
+            if isinstance(img_path,str):
+                out_np = np.asarray(Image.open(img_path))
+            else:
+                out_np = np.asarray(img_path)
+            if(out_np.ndim==2):
+                out_np = np.tile(out_np[:,:,None],3)
+            return out_np
+
+        def resize_img(img, HW=(256,256), resample=3):
+            return np.asarray(Image.fromarray(img).resize((HW[1],HW[0]), resample=resample))
+
+        def preprocess_img(img_rgb_orig, HW=(256,256), resample=3):
+            img_rgb_rs = resize_img(img_rgb_orig, HW=HW, resample=resample)
+
+            img_rgb_orig = np.float32(img_rgb_orig * (1./255))
+            img_lab_orig = cv2.cvtColor(img_rgb_orig, cv2.COLOR_RGB2LAB)
+
+            img_rgb_rs = np.float32(img_rgb_rs * (1./255))
+            img_lab_rs = cv2.cvtColor(img_rgb_rs, cv2.COLOR_BGR2LAB)
+
+            img_l_orig = img_lab_orig[:,:,0]
+            img_l_rs = img_lab_rs[:,:,0]
+
+            tens_orig_l = img_l_orig[None,None,:,:]
+            tens_rs_l = img_l_rs[None,None,:,:]
+
+            return (tens_orig_l, tens_rs_l)
+
+        def postprocess_tens(tens_orig_l, out_ab, mode='bilinear'):
+            HW_orig = tens_orig_l.shape[2:]
+            HW = out_ab.shape[2:]
+
+            if(HW_orig[0]!=HW[0] or HW_orig[1]!=HW[1]):
+                # out_ab_orig = F.interpolate(out_ab, size=HW_orig, mode='bilinear')
+                out_a = cv2.resize(out_ab[0,0,:,:], (HW_orig[1], HW_orig[0]), interpolation=cv2.INTER_LINEAR)
+                out_b = cv2.resize(out_ab[0,1,:,:], (HW_orig[1], HW_orig[0]), interpolation=cv2.INTER_LINEAR)
+                out_ab_orig = np.concatenate(([out_a],[out_b]),axis=0)
+                out_ab_orig = out_ab_orig[None,...]
+            else:
+                out_ab_orig = out_ab
+
+            out_lab_orig = np.concatenate((tens_orig_l, out_ab_orig), axis=1)
+
+            return cv2.cvtColor(out_lab_orig[0,...].transpose((1,2,0)), cv2.COLOR_LAB2RGB)
+
+        input_name = self.model.get_inputs()[0].name
+        output_name = self.model.get_outputs()[0].name
+        img = load_img(data)
+        (tens_l_orig, tens_l_rs) = preprocess_img(img, HW=(256,256))
+        input_data = tens_l_rs
+        output_data = self.model.run([output_name], {input_name: input_data})[0]
+        self.color_res = postprocess_tens(tens_l_orig, output_data)
+        if img_type:
+            self._get_img = img_type
+            if img_type =='cv2':
+                # image = cv2.cvtcolor(image,cv2.BGR2RGB)
+                self.color_res = cv2.cvtColor(self.color_res,cv2.COLOR_BGR2RGB)
+            if show:
+                self.show(self.color_res)
+            return self.color_res,self.color_res
+        else:
+            self._get_img = 'pil'
+        return self.color_res
+
     def _empty_embedding(self):
         return np.empty((0, 512), dtype=np.float32)
 
@@ -499,7 +722,6 @@ class Workflow:
         self.doc = read_squad_examples_context(input_data=context)
         return self.doc 
 
-
     def _nlp_qa_infer(self,data=None,context=None):
         if context is not None:
             self.contexts.append(context)
@@ -565,7 +787,6 @@ class Workflow:
         return res
     
     def _baseml_infer(self,data=None, show=None, img_type=None):
-        print(self.input_shape)
         if data is not None and self.input_shape is not None: 
             model_input_shape = str(self.input_shape).replace(str(self.input_shape[0]), 'batch')
             assert type(self.demo_input) == type(data), f"Error Code: -309. The data type {type(data)} doesn't match the model input type {type(self.demo_input)}. Example input: {self.demo_input.tolist()}."
@@ -678,7 +899,6 @@ class Workflow:
             self._get_img = 'pil'
         return self.style_res
 
-
     def _cls_infer(self,data=None,show=None,img_type=None):
         def preprocess(input_data):
             # convert the input data into the float32 input
@@ -796,7 +1016,12 @@ class Workflow:
 
         result, elapse_list = self.model(data)
         if result is None:
-            return []
+            self.bboxs, self.classes, self.scores = [],[],[]
+            data = cv2.imread(data) if isinstance(data,str) else data
+            if show:
+                self._get_img = 'cv2'
+                self.show(data)
+            return [],data
         self.bboxs, self.classes,    self.scores = list(zip(*result))
         result = list(zip(self.classes,self.bboxs))
         if img_type:
@@ -1062,10 +1287,12 @@ class Workflow:
             img = img[1]
         try:
             if self._get_img.lower() == 'cv2':
-                img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
+                if len(img.shape) == 3:
+                    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
             elif self._get_img.lower() == 'pil':
                 img = img
-            plt.imshow(img)
+            # plt.clf()
+            plt.imshow(img, cmap='gray')
             plt.show()
         except:
             raise Exception(ecf.INFER_BEFORE_SHOW())
@@ -1080,6 +1307,8 @@ class Workflow:
             if isinstance(img,tuple):
                 img = img[1]
             img = img
+            if img.dtype != 'uint8':
+                img = (img*255).astype('uint8')
             a = Image.fromarray(img)
             a.save(save_path)
             # plt.imshow(img)
@@ -1197,6 +1426,8 @@ class Workflow:
             }
         elif self.task in ['gen_style_mosaic','gen_style_candy','gen_style_rain-princess','gen_style_udnie','gen_style_pointilism','gen_style_custom']:
             formalize_result = self.style_res
+        elif self.task in ['gen_color']:
+            formalize_result = self.color_res
         elif self.task in ['nlp_qa']:
             formalize_keys = {
                 "zh":["问题","回答","文本","分数","上下文"],
@@ -1235,6 +1466,18 @@ class Workflow:
                 formalize_keys[language][2]:self.lane_mask,
                 formalize_keys[language][3]:self.area_mask,
             }
+        elif self.task in ['seg_sam']:
+            formalize_keys = {
+                "zh":["掩码","分数"],
+                "en":["masks","scores"],
+                "ru":["маски","баллы"],
+                "de":["Maske","Partituren"],
+                "fr":["masques","partitions"]
+            }
+            formalize_result = {
+                formalize_keys[language][0]:self.masks,
+                formalize_keys[language][1]:self.mask_scores.tolist(),
+            }
         if isprint:
             try:
                 pprint.pprint(formalize_result,sort_dicts=False)
